import os,os.path,string,sys
import cherrypy
import DLFCN
sys.setdlopenflags(DLFCN.RTLD_GLOBAL+DLFCN.RTLD_LAZY)
import coral
from pluginCondDBPyInterface import *
from CondCore.TagCollection import Node,tagInventory,TagTree
from Common import config
import pickle

class GlobalTag(object):
    def __init__(self,configfilepath):
        self.__config=config.Config()
        self.__config.from_ini(configfilepath)
        self.__authpath=self.__config.globaltag.auth_path
        self.__globaltagconfig=self.__config.globaltag.config_file
        self.__servicemap=self.__getServices()
        if len(self.__servicemap)==0:
            raise ValueError('no service is configured')
        
        self.__coralcontext=coral.Context()
        self.__coralcontext.setVerbosityLevel( 'ERROR' )
        self.__svc=coral.ConnectionService(self.__coralcontext)
        self.__svc.configuration().setDefaultAuthenticationService('CORAL/Services/XMLAuthenticationService')
        os.environ['CORAL_AUTH_PATH']=self.__authpath
        #print 'authpath',self.__authpath
        #print 'tns_admin',os.environ['TNS_ADMIN']
        self.__overview={}

    @cherrypy.expose
    def index(self):
        """Globaltag Top level page
        """
        return "GLOBALTAGPAGE!!!"
    
    @cherrypy.expose
    def overview(self):    
        """Overview the inventory and trees in all db instances
           The result is a dictionary: {connectstring:(basetaglist,treenamelist)}
        """
        self.__overview={}
        for dbconnect in self.__servicemap:
            dbconnect='oracle://'+dbconnect
            try:
                session=self.__svc.connect(dbconnect,accessMode=coral.access_Update)
            except :
                print 'unable to connect '+dbconnect+' skip \n'
                continue
        #session=self.__svc.connect(dbconnect,accessMode=coral.access_ReadOnly)
            try: 
                inv=tagInventory.tagInventory(session)
                alltags=inv.getAllEntries()
                #print alltags
                session.transaction().start(True)
                tbs=session.nominalSchema().listTables()
                session.transaction().commit()
                trees=[]
                for tbname in tbs:
                  if tbname.find('TAGTREE_TABLE_')!=-1:
                    treename=self.__gettreename(tbname)
                    trees.append(treename)
                #print trees
            except:
                #if schema is empty or any reading problems, return empty result
                self.__overview[dbconnect]=([],[])
                continue  
            self.__overview[dbconnect]=(alltags,trees)
        #print 'services',self.__overview.keys()
        return pickle.dumps(self.__overview)
    
    def __getServices(self):
        """Get global tag db instance map from config file
        """
        sectionName='GLOBALTAG'
        print 'config file in use ',self.__globaltagconfig
        try:
            globaltagconfig=config.Config()
            globaltagconfig.from_ini(self.__globaltagconfig)
            globaltagproperty=getattr(globaltagconfig,sectionName)
        except AttributeError:
            print 'No configuration found for',sectionName
            return []
        try:
            servicestr=getattr(globaltagproperty,'services')
            servicestr_clean=''.join(servicestr.strip(' ').split(' '))
            return servicestr_clean.split(',')
        except AttributeError:
            print 'services is not configured for',sectionName
            return []
        
    def __serviceToServlet(self,servicename):
        sectionName='GLOBALTAG'
        try:
            globaltagproperty=getattr(self.__config,sectionName)
        except AttributeError:
            print 'No configuration found for',sectionName
            return ''
        try:
            servletname=getattr(globaltagproperty,servicename)
            return servletname.strip(' ')
        except AttributeError:
            print 'servletname is not configured for',servicename
            return ''
    def __gettreename(self,tbname):
        """Given tree table name returns tree name
        """
        treename=tbname.split('_',2)[2]
        return treename
    @cherrypy.expose
    def getIOVTag(self,tag,service,schema,viacache):
        """Get selected iov tag details, require jumping to the real db!
        """
        pass

    @cherrypy.expose
    def getTree(self,treename,service,schema,viacache):
        """Get all nodes of a given tree. deliver result as pickled list
           It's leaf if lft=rgt-1. To be solved at client side
        """
        dbconnect='oracle://'+service+'/'+schema
        session=self.__svc.connect(dbconnect,accessMode=coral.access_Update)
        mytree=TagTree.tagTree(session,treename)
        result=mytree.getSubtree('ROOT')
        return pickle.dumps(result) 
    
    
